/* Generated: Thu Dec 19 2024 10:16:38 GMT-0700 (Mountain Standard Time) */

static void CCONV PhidgetLEDArray_errorHandler(PhidgetChannelHandle ch, Phidget_ErrorEventCode code);
static void CCONV PhidgetLEDArray_free(PhidgetChannelHandle *ch);
static PhidgetReturnCode CCONV PhidgetLEDArray_bridgeInput(PhidgetChannelHandle phid, BridgePacket *bp);
static PhidgetReturnCode CCONV PhidgetLEDArray_setStatus(PhidgetChannelHandle phid, BridgePacket *bp);
static PhidgetReturnCode CCONV PhidgetLEDArray_getStatus(PhidgetChannelHandle phid, BridgePacket **bp);
static PhidgetReturnCode CCONV PhidgetLEDArray_initAfterOpen(PhidgetChannelHandle phid);
static PhidgetReturnCode CCONV PhidgetLEDArray_setDefaults(PhidgetChannelHandle phid);
static void CCONV PhidgetLEDArray_fireInitialEvents(PhidgetChannelHandle phid);
static int CCONV PhidgetLEDArray_hasInitialState(PhidgetChannelHandle phid);

struct _PhidgetLEDArray {
	struct _PhidgetChannel phid;
	Phidget_IOVoltage IOVoltage;
	uint32_t maxSendPacketLength;
};

static PhidgetReturnCode CCONV
_setStatus(PhidgetChannelHandle phid, BridgePacket *bp) {
	PhidgetLEDArrayHandle ch;
	int nsversion;

	ch = (PhidgetLEDArrayHandle)phid;

	nsversion = getBridgePacketUInt32ByName(bp, "_class_version_");
	if (nsversion != 3) {
		loginfo("%"PRIphid": server/client class version mismatch: %d != 3 - functionality may be limited.", phid, nsversion);
	}

	if (hasBridgePacketByName(bp, "IOVoltage"))
		ch->IOVoltage = getBridgePacketInt32ByName(bp, "IOVoltage");
	if (hasBridgePacketByName(bp, "maxSendPacketLength"))
		ch->maxSendPacketLength = getBridgePacketUInt32ByName(bp, "maxSendPacketLength");

	return (EPHIDGET_OK);
}

static PhidgetReturnCode CCONV
_getStatus(PhidgetChannelHandle phid, BridgePacket **bp) {
	PhidgetLEDArrayHandle ch;

	ch = (PhidgetLEDArrayHandle)phid;

	return (createBridgePacket(bp, BP_SETSTATUS, 3, "_class_version_=%u"
	  ",IOVoltage=%d"
	  ",maxSendPacketLength=%u"
	  ,3 /* class version */
	  ,ch->IOVoltage
	  ,ch->maxSendPacketLength
	));
}

static PhidgetReturnCode CCONV
_bridgeInput(PhidgetChannelHandle phid, BridgePacket *bp) {
	PhidgetLEDArrayHandle ch;
	PhidgetReturnCode res;

	ch = (PhidgetLEDArrayHandle)phid;
	res = EPHIDGET_OK;

	switch (bp->vpkt) {
	case BP_DATAOUT:
		res = DEVBRIDGEINPUT(phid, bp);
		break;
	case BP_SETIOVOLTAGE:
		if (!supportedIOVoltage(phid, (Phidget_IOVoltage)getBridgePacketInt32(bp, 0)))
			return (MOS_ERROR(bp->iop, EPHIDGET_INVALIDARG,
			  "Specified IOVoltage is unsupported by this device."));
		res = DEVBRIDGEINPUT(phid, bp);
		if (res != EPHIDGET_OK)
			break;
		ch->IOVoltage = getBridgePacketInt32(bp, 0);
		if (bridgePacketIsFromNet(bp)) {
			FIRE_PROPERTYCHANGE(ch, "IOVoltage");
		}
		break;
	default:
		logerr("%"PRIphid": unsupported bridge packet:0x%x", phid, bp->vpkt);
		res = EPHIDGET_UNSUPPORTED;
	}

	return (res);
}

static PhidgetReturnCode CCONV
_initAfterOpen(PhidgetChannelHandle phid) {
	PhidgetReturnCode ret;

	TESTPTR(phid);

	ret = EPHIDGET_OK;


	switch (phid->UCD->uid) {
	default:
		MOS_PANIC("Unsupported Channel");
	}


	return (ret);
}

static PhidgetReturnCode CCONV
_setDefaults(PhidgetChannelHandle phid) {
	PhidgetReturnCode ret;

	TESTPTR(phid);

	ret = EPHIDGET_OK;

	switch (phid->UCD->uid) {
	default:
		MOS_PANIC("Unsupported Channel");
	}

	return (ret);
}

static void CCONV
_fireInitialEvents(PhidgetChannelHandle phid) {

}

static int CCONV
_hasInitialState(PhidgetChannelHandle phid) {

	return (PTRUE);
}

static void CCONV
_free(PhidgetChannelHandle *ch) {

	mos_free(*ch, sizeof (struct _PhidgetLEDArray));
}

static PhidgetReturnCode CCONV
_create(PhidgetLEDArrayHandle *phidp) {

	CHANNELCREATE_BODY(LEDArray, PHIDCHCLASS_LEDARRAY);
	return (EPHIDGET_OK);
}

API_PRETURN
PhidgetLEDArray_delete(PhidgetLEDArrayHandle *phidp) {

	return (Phidget_delete((PhidgetHandle *)phidp));
}

API_PRETURN
PhidgetLEDArray_setIOVoltage(PhidgetLEDArrayHandle ch, Phidget_IOVoltage IOVoltage) {

	TESTPTR_PR(ch);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_LEDARRAY);
	TESTATTACHED_PR(ch);

	return (bridgeSendToDevice((PhidgetChannelHandle)ch, BP_SETIOVOLTAGE, NULL, NULL, 1, "%d",
	  IOVoltage));
}

API_PRETURN
PhidgetLEDArray_getIOVoltage(PhidgetLEDArrayHandle ch, Phidget_IOVoltage *IOVoltage) {

	TESTPTR_PR(ch);
	TESTPTR_PR(IOVoltage);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_LEDARRAY);
	TESTATTACHED_PR(ch);

	*IOVoltage = ch->IOVoltage;
	if (ch->IOVoltage == (Phidget_IOVoltage)PUNK_ENUM)
		return (PHID_RETURN(EPHIDGET_UNKNOWNVAL));
	return (EPHIDGET_OK);
}

API_PRETURN
PhidgetLEDArray_getMaxSendPacketLength(PhidgetLEDArrayHandle ch, uint32_t *maxSendPacketLength) {

	TESTPTR_PR(ch);
	TESTPTR_PR(maxSendPacketLength);
	TESTCHANNELCLASS_PR(ch, PHIDCHCLASS_LEDARRAY);
	TESTATTACHED_PR(ch);

	*maxSendPacketLength = ch->maxSendPacketLength;
	if (ch->maxSendPacketLength == (uint32_t)PUNK_UINT32)
		return (PHID_RETURN(EPHIDGET_UNKNOWNVAL));
	return (EPHIDGET_OK);
}
